@@ -8,7 +8,8 @@ from meta.mw import MediaWikiContext
 from sidif.sidif import SiDIFParser
 from lodstorage.jsonable import JSONAble
 import sys
-
+import typing
+import urllib3
 
 class MetaModelElement(JSONAble):
     '''
@@ -63,8 +64,8 @@ class Context(MetaModelElement):
         samples = [{
             "name": "MetaModel",
             "since": datetime.strptime("2015-01-23", "%Y-%m-%d"),
-            "copyright": "2015-2022 BITPlan GmbH",
-            "master": "http://master.bitplan.com"
+            "copyright": "2015-2023 BITPlan GmbH",
+            "master": "http://smw-meta.bitplan.com"
         }]
         return samples
     
@@ -195,7 +196,63 @@ true is targetMultiple of it
                 topic=context.lookupTopic(topic_name,f"property {prop.name}")
                 if topic:
                     topic.properties[prop.name] = prop
+        for topic in context.topics.values():
+            topic.setConceptProperty()
         return context
+    
+    @classmethod
+    def fromSiDIF_input(cls,sidif_input:str,debug:bool=False):
+        """
+        initialize me from the given SiDIF input which might be a file path
+        or url
+        
+        Args:
+            sidif_input: path to local file or URL
+            debug(bool): if True swith debugging on
+            
+        Returns:
+            Tuple[Context,str,str]: context, error and errorMessage
+        """
+        if sidif_input.startswith("http:") or sidif_input.startswith("https:"):
+            url=sidif_input
+            http = urllib3.PoolManager()
+            response = http.request('GET', url)
+            sidif = response.data.decode('utf-8')
+        else:
+            sidif_path=sidif_input
+            with open(sidif_path, 'r') as sidif_file:
+                sidif=sidif_file.read()
+        return Context.fromSiDIF(sidif, title=sidif_input, debug=debug)
+    
+    @classmethod
+    def fromSiDIF(cls,sidif:str,title:str,depth:int=None,debug:bool=False)->typing.Tuple['Context',object,str]:
+        """
+        initialize me from the given SiDIF markup
+        
+        Args:
+            sidif(str): the SiDIF markup to parse
+            title(str): the title for the SiDIF
+            depth(int): the explain depth to show for the errorMessage
+            debug(bool): if True handle debugging
+            
+        Returns:
+            Tuple[Context,str,str]: context, error and errorMessage
+            
+        """
+        errMsg = None
+        sp = SiDIFParser(debug=debug)
+        parsed, error = sp.parseText(sidif, title,depth=depth)
+        if debug:
+            sp.printResult(parsed)
+        if error is None:
+            dif = parsed[0]
+            did = dif.toDictOfDicts()
+            context = Context.fromDictOfDicts(did)
+            context.dif=dif
+            context.did=did
+        else:
+            errMsg=sp.errorMessage(title,error, depth=depth)
+        return context, error,errMsg
         
     @classmethod
     def fromWikiContext(cls, mw_context:MediaWikiContext, depth:int=None,debug:bool=False) -> 'MetaModel':
@@ -222,16 +279,7 @@ true is targetMultiple of it
             error = ex
             errMsg=str(ex)
         if sidif is not None:
-            sp = SiDIFParser(debug=debug)
-            parsed, error = sp.parseText(sidif, mw_context.wikiId,depth=depth)
-            if debug:
-                sp.printResult(parsed)
-            if error is None:
-                dif = parsed[0]
-                did = dif.toDictOfDicts()
-                context = Context.fromDictOfDicts(did)
-            else:
-                errMsg=sp.errorMessage(mw_context.wikiId,error, depth=depth)
+            context,error,errMsg=cls.fromSiDIF(sidif=sidif, title=mw_context.wikiId,depth=depth,debug=debug)
         return context, error,errMsg
     
 class Topic(MetaModelElement):
@@ -264,6 +312,22 @@ class Topic(MetaModelElement):
                 }]
         return samples
     
+    def setConceptProperty(self):
+        """ 
+        set my concept property to any primary key or mandatory property
+        """
+        self.conceptProperty=None 
+        for prop in self.properties.values():
+            mandatory=False
+            primaryKey=False
+            if hasattr(prop, "mandatory"):
+                mandatory=prop.mandatory
+            if hasattr(prop,"primaryKey"):
+                primaryKey=prop.primaryKey
+            if mandatory or primaryKey:
+                self.conceptProperty=prop
+                break
+    
     def getPluralName(self) -> str:
         """
         get the plural name for this topic
